/*
 * This file is a part of TTCalc - a mathematical calculator
 * and is distributed under the (new) BSD licence.
 * Author: Tomasz Sowa <t.sowa@slimaczek.pl>
 */

/* 
 * Copyright (c) 2006-2007, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *    
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *    
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "compileconfig.h"

#include "languages.h"






const char * Languages::ErrorMessage(Country country, ttmath::ErrorCode code)
{
const char * unk_err = "unknown error";

	if( error_messages_tab.empty() )
		InitErrorMessagesTab();

	unsigned int cid = static_cast<unsigned int>( country );
	if( cid >= error_messages_tab.size() )
		return unk_err;

	std::map<ttmath::ErrorCode, std::string>::const_iterator i;

	i = error_messages_tab[cid].find( code );
	if( i == error_messages_tab[cid].end() )
		return unk_err;

return i->second.c_str();
}


const char * Languages::ErrorMessage(ttmath::ErrorCode code)
{
	return ErrorMessage(current_country, code);
}



void Languages::InsertErrorPair(ttmath::ErrorCode code, const char * message)
{
	if( error_messages_tab.empty() )
		return;

	error_messages_tab.back().insert( std::make_pair(code, message) );
}


void Languages::InitErrorMessagesTab()
{
	error_messages_tab.push_back( std::map<ttmath::ErrorCode, std::string>() );

	InsertErrorPair(ttmath::err_ok,"ok");
	InsertErrorPair(ttmath::err_nothing_has_read,"");
	InsertErrorPair(ttmath::err_unknown_character,"Unknown character");
	InsertErrorPair(ttmath::err_unexpected_final_bracket,"An unexpected final bracket");
	InsertErrorPair(ttmath::err_stack_not_clear,"An unknown character has left");
	InsertErrorPair(ttmath::err_unknown_variable,"An unknown variable");
	InsertErrorPair(ttmath::err_division_by_zero,"Division by zero");
	InsertErrorPair(ttmath::err_interrupt,"?");
	InsertErrorPair(ttmath::err_overflow,"Overflow");
	InsertErrorPair(ttmath::err_unknown_function,"An unknown function");
	InsertErrorPair(ttmath::err_unexpected_semicolon_operator,"An unexpected semicolon operator");
	InsertErrorPair(ttmath::err_improper_amount_of_arguments,"Improper amount of arguments");
	InsertErrorPair(ttmath::err_improper_argument,"Improper argument");
	InsertErrorPair(ttmath::err_unexpected_end,"Unexpected end");
	InsertErrorPair(ttmath::err_internal_error,"An internal error");

	InsertErrorPair(ttmath::err_incorrect_name,"Incorrect name of a variable or function");
	InsertErrorPair(ttmath::err_incorrect_value,"Incorrect value of a variable or function");
	InsertErrorPair(ttmath::err_variable_exists,"This variable already exists");
	InsertErrorPair(ttmath::err_variable_loop,"There's a recurrence between variables");
	InsertErrorPair(ttmath::err_functions_loop,"There's a recurrence between functions");
	InsertErrorPair(ttmath::err_must_be_only_one_value,"Variables or functions must return only one value");
	InsertErrorPair(ttmath::err_still_calculating,"Calculating...");


// pl

	error_messages_tab.push_back( std::map<ttmath::ErrorCode, std::string>() );

	InsertErrorPair(ttmath::err_ok,"ok");
	InsertErrorPair(ttmath::err_nothing_has_read,"");
	InsertErrorPair(ttmath::err_unknown_character,"Nieznany znak");
	InsertErrorPair(ttmath::err_unexpected_final_bracket,"Nieoczekiwany nawias zamykajcy");
	InsertErrorPair(ttmath::err_stack_not_clear,"Pozosta nieznany znak");
	InsertErrorPair(ttmath::err_unknown_variable,"Nieznana zmienna");
	InsertErrorPair(ttmath::err_division_by_zero,"Dzielenie przez zero");
	InsertErrorPair(ttmath::err_interrupt,"?");
	InsertErrorPair(ttmath::err_overflow,"Przekroczony zakres");
	InsertErrorPair(ttmath::err_unknown_function,"Nieznana funkcja");
	InsertErrorPair(ttmath::err_unexpected_semicolon_operator,"Nieoczekiwany operator 'rednik'");
	InsertErrorPair(ttmath::err_improper_amount_of_arguments,"Niewaciwa liczba argumentw");
	InsertErrorPair(ttmath::err_improper_argument,"Niewaciwy argument");
	InsertErrorPair(ttmath::err_unexpected_end,"Nieoczekiwany koniec");
	InsertErrorPair(ttmath::err_internal_error,"Bd wewntrzny programu!");

	InsertErrorPair(ttmath::err_incorrect_name,"Nieprawidowa nazwa zmiennej lub funkcji");
	InsertErrorPair(ttmath::err_incorrect_value,"Nieprawidowa warto zmiennej lub funkcji");
	InsertErrorPair(ttmath::err_variable_exists,"Ta zmienna juz istnieje");
	InsertErrorPair(ttmath::err_variable_loop,"Pomidzy zmiennymi zachodzi wywoanie rekurencyjne");
	InsertErrorPair(ttmath::err_functions_loop,"Pomidzy funkcjami zachodzi wywoanie rekurencyjne");
	InsertErrorPair(ttmath::err_must_be_only_one_value,"Zmienne albo funkcje mog posiada (zwraca) tylko jedn warto");
	InsertErrorPair(ttmath::err_still_calculating,"Obliczanie...");



	

}



// for gui messages




const char * Languages::GuiMessage(Country country, GuiMsg code)
{
const char * unk_msg = "unknown";

	if( gui_messages_tab.empty() )
		InitGuiMessagesTab();

	unsigned int cid = static_cast<unsigned int>( country );
	if( cid >= gui_messages_tab.size() )
		return unk_msg;

	std::map<GuiMsg, std::string>::const_iterator i;

	i = gui_messages_tab[cid].find( code );
	if( i == gui_messages_tab[cid].end() )
		return unk_msg;

return i->second.c_str();
}

const char * Languages::GuiMessage(GuiMsg code)
{
	return GuiMessage(current_country, code);
}
void Languages::InsertGuiPair(GuiMsg code, const char * message)
{
	if( gui_messages_tab.empty() )
		return;

	gui_messages_tab.back().insert( std::make_pair(code, message) );
}


void Languages::InitGuiMessagesTab()
{
	// en
	gui_messages_tab.push_back( std::map<GuiMsg, std::string>() );

	InsertGuiPair(button_ok,"Ok");
	InsertGuiPair(button_cancel,"Cancel");
	InsertGuiPair(message_box_caption,"TTCalc");

	InsertGuiPair(dialog_box_add_variable_caption,"Add a new variable");
	InsertGuiPair(dialog_box_edit_variable_caption,"Edit a variable");
	InsertGuiPair(dialog_box_add_variable_incorrect_name,"Incorrect name of variable");
	InsertGuiPair(dialog_box_add_variable_incorrect_value,"Incorrect value of variable");
	InsertGuiPair(dialog_box_add_variable_variable_exists,"This variable already exists");
	InsertGuiPair(dialog_box_edit_variable_unknown_variable,"There isn't this variable in my table. There's probably an internal error!");
	InsertGuiPair(dialog_box_delete_variable_confirm,"Are you sure that you want to delete those variables?");
	InsertGuiPair(dialog_box_variable_not_all_deleted,"There are some variables which I was not able to delete. Probably an internal error!");
	InsertGuiPair(dialog_box_add_function_caption,"Add a new function");
	InsertGuiPair(dialog_box_edit_function_caption,"Edit a function");
	InsertGuiPair(dialog_box_add_function_function_exists,"This function already exists");
	InsertGuiPair(dialog_box_edit_function_unknown_function,"There isn't this function in my table. There's probably an internal error!");
	InsertGuiPair(dialog_box_delete_function_confirm,"Are you sure that you want to delete those functions?");
	InsertGuiPair(dialog_box_function_not_all_deleted,"There are some functions which I was not able to delete. Probably an internal error!");

	InsertGuiPair(list_variables_header_1,"Name");
	InsertGuiPair(list_variables_header_2,"Value");
	InsertGuiPair(list_functions_header_1,"Name");
	InsertGuiPair(list_functions_header_2,"Param.");
	InsertGuiPair(list_functions_header_3,"Value");

	InsertGuiPair(button_add,"Add");
	InsertGuiPair(button_edit,"Edit");
	InsertGuiPair(button_delete,"Delete");
	InsertGuiPair(button_clear,"C");
	InsertGuiPair(tab_standard,"Standard");
	InsertGuiPair(tab_variables,"Variables");
	InsertGuiPair(tab_functions,"Functions");
	InsertGuiPair(tab_precision,"Precision");
	InsertGuiPair(tab_display,"Display");

	InsertGuiPair(radio_precision_1,"Small - 96 bits for mantissa, 32 bits for exponent");
	InsertGuiPair(radio_precision_2,"Medium - 192 bits for mantissa, 64 bits for exponent");
	InsertGuiPair(radio_precision_3,"Big - 288 bits for mantissa, 96 bits for exponent");

	InsertGuiPair(precision_1_info,"(+/-)6.9 e+646457021, 26 valid digits (decimal)");
	InsertGuiPair(precision_2_info,"(+/-)4.3 e+2776511644261678623, 56 valid digits");
	InsertGuiPair(precision_3_info,"(+/-)2.5 e+11925026709067095507636213441, 85 valid digits");

	InsertGuiPair(overflow_during_printing,"Overflow during printing");

	InsertGuiPair(combo_rounding_none,"None");
	InsertGuiPair(combo_rounding_cut_last_digits,"Cut off last non-valid digits");
	InsertGuiPair(combo_rounding_integer,"Round to the nearest integer");
	InsertGuiPair(combo_rounding_to_number,"to");
	InsertGuiPair(combo_rounding_after_comma,"digit(s) after comma");
	InsertGuiPair(display_input,			"Input");
	InsertGuiPair(display_output,			"Output");
	InsertGuiPair(display_rounding,			"Rounding");
	InsertGuiPair(display_always_scientific,"Always");
	InsertGuiPair(display_not_always_scientific,"When the exp is greater than:");
	InsertGuiPair(display_digit,			"digits");
	InsertGuiPair(display_group_scientific,	"Print the result as the scientific value");

	InsertGuiPair(menu_view,				"&View");
	InsertGuiPair(menu_edit,				"&Edit");
	InsertGuiPair(menu_help,				"&Help");
	InsertGuiPair(menu_language,			"&Language");
	InsertGuiPair(menu_view_new_window,		"&New window");
	InsertGuiPair(menu_view_normal_view,	"No&rmal view");
	InsertGuiPair(menu_view_compact_view,	"C&ompact view");
	InsertGuiPair(menu_view_always_on_top,	"&Always on top");
	InsertGuiPair(menu_view_lang_english,	"&English");
	InsertGuiPair(menu_view_lang_polish,	"&Polish");
	InsertGuiPair(menu_view_close_program,	"&Close");
	InsertGuiPair(menu_view_edit_undo,		"&Undo");
	InsertGuiPair(menu_view_edit_paste,		"&Paste");
	InsertGuiPair(menu_view_edit_copy_result,"&Copy the result");
	InsertGuiPair(menu_view_help_about,		"&About");
	InsertGuiPair(cant_init_calculations,	"I could not initialize the module of calculations");
	InsertGuiPair(message_box_error_caption,"TTCalc");
	InsertGuiPair(cant_create_thread,		"I could not create the second thread for calculating");
	InsertGuiPair(cant_create_main_window,	"I could not create the main window of the application");
	InsertGuiPair(cant_init_common_controls,"I could not initialize the common controls (InitCommonControlsEx)");
	InsertGuiPair(about_text,
		"Mathematical calculator TTCalc %d.%d.%d\r\n"
		"Author: Tomasz Sowa\r\n"
		"Contact: t.sowa@slimaczek.pl\r\n"
		"Licence: (New) BSD licence\r\n"
		"Project page: http://sourceforge.net/projects/ttcalc\r\n"
		"Mathemathical library: TTMath %d.%d.%d\r\n"
		"Programming language: C++\r\n"
		"Compiler: %s\r\n"
		"\r\n"
		"This program uses the TTMath mathematical library"
		" which can be found at http://sourceforge.net/projects/ttmath\r\n"
		"\r\n"
		"If you have any questions, advices or interesting ideas about"
		" this program or if you want to join to this project as"
		" the developer or programmer just contact with me."
		);
	InsertGuiPair(about_box_title,			"About");



	// pl
	gui_messages_tab.push_back( std::map<GuiMsg, std::string>() );

	InsertGuiPair(button_ok,"Ok");
	InsertGuiPair(button_cancel,"Anuluj");
	InsertGuiPair(message_box_caption,"TTCalc");

	InsertGuiPair(dialog_box_add_variable_caption,"Dodaj now zmienn");
	InsertGuiPair(dialog_box_edit_variable_caption,"Zmie warto zmiennej");
	InsertGuiPair(dialog_box_add_variable_incorrect_name,"Nie prawidowa nazwa zmiennej");
	InsertGuiPair(dialog_box_add_variable_incorrect_value,"Nie prawidowa warto zmiennej");
	InsertGuiPair(dialog_box_add_variable_variable_exists,"Podana zmienna ju istnieje");
	InsertGuiPair(dialog_box_edit_variable_unknown_variable,"Podanej zmiennej nie ma w tablicy. Prawdopodobnie bd wewntrzny programu.");
	InsertGuiPair(dialog_box_delete_variable_confirm,"Czy napewno usun zaznaczone zmienne?");
	InsertGuiPair(dialog_box_variable_not_all_deleted,"Zostao kilka zmiennych ktrych nie mona byo skasowa. Prawdopodobnie bd wewntrzny programu.");
	InsertGuiPair(dialog_box_add_function_caption,"Dodaj now funkcj");
	InsertGuiPair(dialog_box_edit_function_caption,"Zmie warto funkcji");
	InsertGuiPair(dialog_box_add_function_function_exists,"Podana funkcja ju istnieje");
	InsertGuiPair(dialog_box_edit_function_unknown_function,"Podanej funkcji nie ma w tablicy. Prawdopodobnie bd wewntrzny programu.");
	InsertGuiPair(dialog_box_delete_function_confirm,"Czy napewno usun zaznaczone funkcje?");
	InsertGuiPair(dialog_box_function_not_all_deleted,"Zostao kilka funkcji ktrych nie mona byo skasowa. Prawdopodobnie bd wewntrzny programu.");

	InsertGuiPair(list_variables_header_1,"Nazwa");
	InsertGuiPair(list_variables_header_2,"Warto");
	InsertGuiPair(list_functions_header_1,"Nazwa");
	InsertGuiPair(list_functions_header_2,"Parametry");
	InsertGuiPair(list_functions_header_3,"Warto");

	InsertGuiPair(button_add,"Dodaj");
	InsertGuiPair(button_edit,"Edytuj");
	InsertGuiPair(button_delete,"Usu");
	InsertGuiPair(button_clear,"C");
	InsertGuiPair(tab_standard,"Standard");
	InsertGuiPair(tab_variables,"Zmienne");
	InsertGuiPair(tab_functions,"Funkcje");
	InsertGuiPair(tab_precision,"Precyzja");
	InsertGuiPair(tab_display,"Wywietlanie");

	InsertGuiPair(radio_precision_1,"Maa - 96 bitowa mantysa, 32 bitowy wykadnik");
	InsertGuiPair(radio_precision_2,"rednia - 192 bitowa mantysa, 64 bitowy wykadnik");
	InsertGuiPair(radio_precision_3,"Dua - 288 bitowa mantysa, 96 bitowy wykadnik");

	InsertGuiPair(precision_1_info,"(+/-)6.9 e+646457021, 26 cyfr znaczcych (w rozwiniciu dziesitnym)");
	InsertGuiPair(precision_2_info,"(+/-)4.3 e+2776511644261678623, 56 cyfr znaczcych");
	InsertGuiPair(precision_3_info,"(+/-)2.5 e+11925026709067095507636213441, 85 cyfr znaczcych");

	InsertGuiPair(overflow_during_printing,"Przepenienie podczas wypisywania");

	InsertGuiPair(combo_rounding_none,"bez zaokrglania");
	InsertGuiPair(combo_rounding_cut_last_digits,"skasowa nie znaczce zera");
	InsertGuiPair(combo_rounding_integer,"do najbliszej cakowitej");
	InsertGuiPair(combo_rounding_to_number,"do");
	InsertGuiPair(combo_rounding_after_comma,"cyfr(y) po przecinku");

	InsertGuiPair(display_input,"Wejcie");
	InsertGuiPair(display_output,"Wyjcie");
	InsertGuiPair(display_rounding,"Zaokrglenie");
	InsertGuiPair(display_always_scientific,"Zawsze");
	InsertGuiPair(display_not_always_scientific,"Jeli eksponent wikszy ni");
	InsertGuiPair(display_digit,			"cyfr");
	InsertGuiPair(display_group_scientific,	"Wywietl wynik w postaci naukowej");

	InsertGuiPair(menu_view,				"&Widok");
	InsertGuiPair(menu_edit,				"&Edycja");
	InsertGuiPair(menu_help,				"&Pomoc");
	InsertGuiPair(menu_language,			"&Jzyk");
	InsertGuiPair(menu_view_new_window,		"&Nowe okno");
	InsertGuiPair(menu_view_normal_view,	"Widok no&rmalny");
	InsertGuiPair(menu_view_compact_view,	"Widok &kompaktowy");
	InsertGuiPair(menu_view_always_on_top,	"Zawsze na &wierzchu");
	InsertGuiPair(menu_view_lang_english,	"Jzyk &angielski");
	InsertGuiPair(menu_view_lang_polish,	"Jzyk &polski");
	InsertGuiPair(menu_view_close_program,	"&Zamknij");
	InsertGuiPair(menu_view_edit_undo,		"&Cofnij");
	InsertGuiPair(menu_view_edit_paste,		"&Wklej");
	InsertGuiPair(menu_view_edit_copy_result,"&Kopiuj wynik");
	InsertGuiPair(menu_view_help_about,		"&O programie");
	InsertGuiPair(cant_init_calculations,	"Nie udao si zainicjalizowa moduu obsugi oblicze");
	InsertGuiPair(message_box_error_caption,"TTCalc");
	InsertGuiPair(cant_create_thread,		"Nie udao si utworzy drugiego wtku do oblicze");
	InsertGuiPair(cant_create_main_window,	"Nie udao si utworzy gwnego okna aplikacji");
	InsertGuiPair(cant_init_common_controls,"Nie udao si zainicjalizowa obsugi Common Controls (InitCommonControlsEx)");
	InsertGuiPair(about_text,
		"Kalkulator matematyczny TTCalc %d.%d.%d\r\n"
		"Autor: Tomasz Sowa\r\n"
		"Kontakt: t.sowa@slimaczek.pl\r\n"
		"Licencja: (New) BSD\r\n"
		"Strona projektu: http://sourceforge.net/projects/ttcalc\r\n"
		"Biblioteka matematyczna: TTMath %d.%d.%d\r\n"
		"Jzyk programowania: C++\r\n"
		"Kompilator: %s\r\n"
		"\r\n"
		"Ten program uywa biblioteki matematycznej TTMath"
		" ktra jest dostpna na http://sourceforge.net/projects/ttmath\r\n"
		"\r\n"
		"Jeeli masz jakie pytania, rady, ciekawe pomysy dotyczce"
		" tego programu lub chciaby doczy jako projektant/programista"
		" poprostu skontaktuj si ze mn."
		);
	InsertGuiPair(about_box_title,			"O programie");

}

